@@ -59,6 +59,8 @@ void get_current_song_info (int argc, char * * argv);
 void get_volume (int, char * *);
 void set_volume (int, char * *);
 
+void select_displayed (int, char * *);
+void select_playing (int, char * *);
 void playlist_position (int, char * *);
 void playlist_advance (int, char * *);
 void playlist_auto_advance_status (int, char * *);
@@ -114,6 +116,8 @@ void playback_stopped (int, char * *);
 void playback_status (int, char * *);
 void playback_seek (int, char * *);
 void playback_seek_relative (int, char * *);
+void playback_record (int, char * *);
+void playback_recording (int, char * *);
 
 void mainwin_show (int, char * *);
 void show_preferences_window (int, char * *);
@@ -123,6 +127,8 @@ void shutdown_audacious_server (int, char * *);
 void show_about_window (int, char * *);
 
 void get_version (int argc, char * * argv);
+void plugin_is_enabled (int argc, char * * argv);
+void plugin_enable (int argc, char * * argv);
 
 void equalizer_get_eq (int argc, char * * argv);
 void equalizer_get_eq_preamp (int argc, char * * argv);
