@@ -33,6 +33,10 @@
 #include "internal.h"
 #include "runtime.h"
 
+#define MAX_POW10 9
+static const unsigned int_pow10[MAX_POW10 + 1] =
+ {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000, 1000000000};
+
 static const char ascii_to_hex[256] =
     "\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0"
     "\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0"
@@ -51,7 +55,13 @@ static const char uri_legal_table[256] =
     "\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0"
     "\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0"
     "\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x0\x1\x1\x1"  // '-' '.' '/'
+#ifdef _WIN32
+    /* We assume ':' is used with a "reserved purpose" (i.e. drive letter).
+     * This assumption might need to be reconsidered for non-file URIs. */
+    "\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x0\x0\x0\x0\x0"  // 0-9 ':'
+#else
     "\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x0\x0\x0\x0\x0\x0"  // 0-9
+#endif
     "\x0\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1"  // A-O
     "\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x0\x0\x0\x0\x1"  // P-Z '_'
     "\x0\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1\x1"  // a-o
@@ -68,6 +78,12 @@ static const char swap_case[256] =
 #define IS_LEGAL(c)  (uri_legal_table[(unsigned char) (c)])
 #define SWAP_CASE(c) (swap_case[(unsigned char) (c)])
 
+#ifdef _WIN32
+#define IS_SEP(c) ((c) == '/' || (c) == '\\')
+#else
+#define IS_SEP(c) ((c) == '/')
+#endif
+
 /* strcmp() that handles nullptr safely */
 EXPORT int strcmp_safe (const char * a, const char * b, int len)
 {
@@ -184,13 +200,13 @@ EXPORT unsigned str_calc_hash (const char * s)
     while (len >= 8)
     {
         h = h * 1954312449 +
-            s[0] * 3963737313 +
-            s[1] * 1291467969 +
-            s[2] * 39135393 +
-            s[3] * 1185921 +
-            s[4] * 35937 +
-            s[5] * 1089 +
-            s[6] * 33 +
+            (unsigned) s[0] * 3963737313 +
+            (unsigned) s[1] * 1291467969 +
+            (unsigned) s[2] * 39135393 +
+            (unsigned) s[3] * 1185921 +
+            (unsigned) s[4] * 35937 +
+            (unsigned) s[5] * 1089 +
+            (unsigned) s[6] * 33 +
             s[7];
 
         s += 8;
@@ -200,9 +216,9 @@ EXPORT unsigned str_calc_hash (const char * s)
     if (len >= 4)
     {
         h = h * 1185921 +
-            s[0] * 35937 +
-            s[1] * 1089 +
-            s[2] * 33 +
+            (unsigned) s[0] * 35937 +
+            (unsigned) s[1] * 1089 +
+            (unsigned) s[2] * 33 +
             s[3];
 
         s += 4;
@@ -304,6 +320,37 @@ EXPORT StringBuf str_tolower_utf8 (const char * str)
     return buf;
 }
 
+EXPORT StringBuf str_toupper (const char * str)
+{
+    StringBuf buf (strlen (str));
+    char * set = buf;
+
+    while (* str)
+        * set ++ = g_ascii_toupper (* str ++);
+
+    return buf;
+}
+
+EXPORT StringBuf str_toupper_utf8 (const char * str)
+{
+    StringBuf buf (6 * strlen (str));
+    char * set = buf;
+    gunichar c;
+
+    while ((c = g_utf8_get_char (str)))
+    {
+        if (c < 128)
+            * set ++ = g_ascii_toupper (c);
+        else
+            set += g_unichar_to_utf8 (g_unichar_toupper (c), set);
+
+        str = g_utf8_next_char (str);
+    }
+
+    buf.resize (set - buf);
+    return buf;
+}
+
 EXPORT void str_replace_char (char * string, char old_c, char new_c)
 {
     while ((string = strchr (string, old_c)))
@@ -347,7 +394,7 @@ EXPORT StringBuf str_decode_percent (const char * str, int len)
     return buf;
 }
 
-/* Percent-encodes <len> bytes of <str>.  If <len> is negative, decodes all of <str>. */
+/* Percent-encodes <len> bytes of <str>.  If <len> is negative, encodes all of <str>. */
 
 EXPORT StringBuf str_encode_percent (const char * str, int len)
 {
@@ -415,6 +462,74 @@ EXPORT StringBuf filename_normalize (StringBuf && filename)
     return std::move (filename);
 }
 
+/* note #1: recommended order is filename_contract(filename_normalize(f)) */
+/* note #2: currently assumes filename is UTF-8 (intended for display) */
+EXPORT StringBuf filename_contract (StringBuf && filename)
+{
+    /* replace home folder with '~' */
+    const char * home = get_home_utf8 ();
+    int homelen = home ? strlen (home) : 0;
+
+    if (homelen && ! strncmp (filename, home, homelen) &&
+     (! filename[homelen] || IS_SEP (filename[homelen])))
+    {
+        filename[0] = '~';
+        filename.remove (1, homelen - 1);
+    }
+
+    return std::move (filename);
+}
+
+/* note #1: recommended order is filename_normalize(filename_expand(f)) */
+/* note #2: currently assumes filename is UTF-8 (intended for display) */
+EXPORT StringBuf filename_expand (StringBuf && filename)
+{
+    /* expand leading '~' */
+    if (filename[0] == '~' && (! filename[1] || IS_SEP(filename[1])))
+    {
+        const char * home = get_home_utf8 ();
+
+        if (home && home[0])
+        {
+            filename[0] = home[0];
+            filename.insert (1, home + 1, -1);
+        }
+    }
+
+    return std::move (filename);
+}
+
+EXPORT StringBuf filename_get_parent (const char * filename)
+{
+    StringBuf buf = filename_normalize (str_copy (filename));
+    const char * base = last_path_element (buf);
+
+    if (! base)
+        return StringBuf ();
+
+#ifdef _WIN32
+    if (base - buf > 3) /* leave "C:\" */
+#else
+    if (base - buf > 1) /* leave leading "/" */
+#endif
+        buf.resize (base - buf - 1);
+    else
+        buf.resize (base - buf);
+
+    return buf;
+}
+
+EXPORT StringBuf filename_get_base (const char * filename)
+{
+    StringBuf buf = filename_normalize (str_copy (filename));
+    const char * base = last_path_element (buf);
+
+    if (base)
+        buf.remove (0, base - buf);
+
+    return buf;
+}
+
 EXPORT StringBuf filename_build (const std::initializer_list<const char *> & elems)
 {
     StringBuf str (-1);
@@ -423,25 +538,14 @@ EXPORT StringBuf filename_build (const std::initializer_list<const char *> & ele
 
     for (const char * s : elems)
     {
-#ifdef _WIN32
-        if (set > str && set[-1] != '/' && set[-1] != '\\')
+        if (set > str && ! IS_SEP (set[-1]))
         {
             if (! left)
                 throw std::bad_alloc ();
 
-            * set ++ = '\\';
+            * set ++ = G_DIR_SEPARATOR;
             left --;
         }
-#else
-        if (set > str && set[-1] != '/')
-        {
-            if (! left)
-                throw std::bad_alloc ();
-
-            * set ++ = '/';
-            left --;
-        }
-#endif
 
         int len = strlen (s);
         if (len > left)
@@ -516,6 +620,14 @@ EXPORT StringBuf uri_to_filename (const char * uri, bool use_locale)
     }
 #endif
 
+    /* if UTF-8 was requested, make sure the result is valid */
+    if (! use_locale)
+    {
+        buf.steal (str_to_utf8 (std::move (buf)));
+        if (! buf)
+            return StringBuf ();
+    }
+
     return filename_normalize (std::move (buf));
 }
 
@@ -524,6 +636,8 @@ EXPORT StringBuf uri_to_filename (const char * uri, bool use_locale)
 
 EXPORT StringBuf uri_to_display (const char * uri)
 {
+    if (! strncmp (uri, "stdin://", 8))
+        return str_copy (_("Standard input"));
     if (! strncmp (uri, "cdda://?", 8))
         return str_printf (_("Audio CD, track %s"), uri + 8);
 
@@ -531,19 +645,10 @@ EXPORT StringBuf uri_to_display (const char * uri)
     if (! buf)
         return str_copy (_("(character encoding error)"));
 
-    if (strncmp (buf, URI_PREFIX, URI_PREFIX_LEN))
-        return buf;
-
-    buf.remove (0, URI_PREFIX_LEN);
-    buf.steal (filename_normalize (std::move (buf)));
-
-    const char * home = get_home_utf8 ();
-    int homelen = home ? strlen (home) : 0;
-
-    if (homelen && ! strncmp (buf, home, homelen) && buf[homelen] == G_DIR_SEPARATOR)
+    if (! strncmp (buf, URI_PREFIX, URI_PREFIX_LEN))
     {
-        buf[0] = '~';
-        buf.remove (1, homelen - 1);
+        buf.remove (0, URI_PREFIX_LEN);
+        return filename_contract (filename_normalize (std::move (buf)));
     }
 
     return buf;
@@ -609,7 +714,7 @@ EXPORT StringBuf uri_get_extension (const char * uri)
 /* Constructs a full URI given:
  *   1. path: one of the following:
  *     a. a full URI (returned unchanged)
- *     b. an absolute filename (in the system locale)
+ *     b. an absolute filename (in UTF-8 or the system locale)
  *     c. a relative path (character set detected according to user settings)
  *   2. reference: the full URI of the playlist containing <path> */
 
@@ -621,7 +726,7 @@ EXPORT StringBuf uri_construct (const char * path, const char * reference)
 
     /* absolute filename */
 #ifdef _WIN32
-    if (path[0] && path[1] == ':' && path[2] == '\\')
+    if (path[0] && path[1] == ':' && IS_SEP (path[2]))
 #else
     if (path[0] == '/')
 #endif
@@ -826,35 +931,61 @@ EXPORT StringBuf index_to_str_list (const Index<String> & index, const char * se
  * have an accuracy of 6 decimal places.
  */
 
+static unsigned str_to_uint (const char * string, const char * * end = nullptr,
+ const char * stop = nullptr)
+{
+    unsigned val = 0;
+    for (char c; string != stop && (c = * string) >= '0' && c <= '9'; string ++)
+        val = val * 10 + (c - '0');
+
+    if (end)
+        * end = string;
+
+    return val;
+}
+
+static int digits_for (unsigned val)
+{
+    int digits = 1;
+
+    for (; val >= 1000; val /= 1000)
+        digits += 3;
+    for (; val >= 10; val /= 10)
+        digits ++;
+
+    return digits;
+}
+
+static void uint_to_str (unsigned val, char * buf, int digits)
+{
+    for (char * rev = buf + digits; rev > buf; val /= 10)
+        * (-- rev) = '0' + val % 10;
+}
+
 EXPORT int str_to_int (const char * string)
 {
     bool neg = (string[0] == '-');
-    if (neg)
+    if (neg || string[0] == '+')
         string ++;
 
-    int val = 0;
-    char c;
-
-    while ((c = * string ++) && c >= '0' && c <= '9')
-        val = val * 10 + (c - '0');
-
+    unsigned val = str_to_uint (string);
     return neg ? -val : val;
 }
 
 EXPORT double str_to_double (const char * string)
 {
     bool neg = (string[0] == '-');
-    if (neg)
+    if (neg || string[0] == '+')
         string ++;
 
-    double val = str_to_int (string);
-    const char * p = strchr (string, '.');
+    const char * p;
+    double val = str_to_uint (string, & p);
 
-    if (p)
+    if (* (p ++) == '.')
     {
-        char buf[7] = "000000";
-        memcpy (buf, p + 1, strlen_bounded (p + 1, 6));
-        val += (double) str_to_int (buf) / 1000000;
+        const char * end;
+        double decimal = str_to_uint (p, & end, p + MAX_POW10);
+        val += decimal / int_pow10[end - p];
     }
 
     return neg ? -val : val;
@@ -863,26 +994,18 @@ EXPORT double str_to_double (const char * string)
 EXPORT StringBuf int_to_str (int val)
 {
     bool neg = (val < 0);
+    unsigned absval = neg ? -val : val;
+
+    int digits = digits_for (absval);
+    StringBuf buf ((neg ? 1 : 0) + digits);
+
+    char * set = buf;
     if (neg)
-        val = -val;
+        * (set ++) = '-';
 
-    char buf[16];
-    char * rev = buf + sizeof buf;
+    uint_to_str (absval, set, digits);
 
-    while (rev > buf)
-    {
-        * (-- rev) = '0' + val % 10;
-        if (! (val /= 10))
-            break;
-    }
-
-    if (neg && rev > buf)
-        * (-- rev) = '-';
-
-    int len = buf + sizeof buf - rev;
-    StringBuf buf2 (len);
-    memcpy (buf2, rev, len);
-    return buf2;
+    return buf;
 }
 
 EXPORT StringBuf double_to_str (double val)
@@ -891,8 +1014,8 @@ EXPORT StringBuf double_to_str (double val)
     if (neg)
         val = -val;
 
-    int i = floor (val);
-    int f = round ((val - i) * 1000000);
+    unsigned i = floor (val);
+    unsigned f = round ((val - i) * 1000000);
 
     if (f == 1000000)
     {
@@ -900,15 +1023,26 @@ EXPORT StringBuf double_to_str (double val)
         f = 0;
     }
 
-    StringBuf buf = str_printf ("%s%d.%06d", neg ? "-" : "", i, f);
+    int decimals = f ? 6 : 0;
+    for (; decimals && ! (f % 10); f /= 10)
+        decimals --;
 
-    char * c = buf + buf.len ();
-    while (c[-1] == '0')
-        c --;
-    if (c[-1] == '.')
-        c --;
+    int digits = digits_for (i);
+    StringBuf buf ((neg ? 1 : 0) + digits + (decimals ? 1 : 0) + decimals);
+
+    char * set = buf;
+    if (neg)
+        * (set ++) = '-';
+
+    uint_to_str (i, set, digits);
+
+    if (decimals)
+    {
+        set += digits;
+        * (set ++) = '.';
+        uint_to_str (f, set, decimals);
+    }
 
-    buf.resize (c - buf);
     return buf;
 }
 
@@ -960,15 +1094,20 @@ EXPORT StringBuf double_array_to_str (const double * array, int count)
 
 EXPORT StringBuf str_format_time (int64_t milliseconds)
 {
+    bool neg = milliseconds < 0;
+
+    if (neg)
+        milliseconds *= -1;
+
     int hours = milliseconds / 3600000;
-    int minutes = (milliseconds / 60000) % 60;
+    int minutes = milliseconds / 60000;
     int seconds = (milliseconds / 1000) % 60;
 
-    if (hours)
-        return str_printf ("%d:%02d:%02d", hours, minutes, seconds);
+    if (hours && aud_get_bool (nullptr, "show_hours"))
+        return str_printf ("%s%d:%02d:%02d", neg ? "- " : "",  hours, minutes % 60, seconds);
     else
     {
         bool zero = aud_get_bool (nullptr, "leading_zero");
-        return str_printf (zero ? "%02d:%02d" : "%d:%02d", minutes, seconds);
+        return str_printf (zero ? "%s%02d:%02d" : "%s%d:%02d", neg ? "- " : "", minutes, seconds);
     }
 }
