@@ -97,21 +97,12 @@ void playback_stop (bool exiting = false);
 
 bool playback_check_serial (int serial);
 void playback_set_info (int entry, Tuple && tuple);
-void playback_setup_decode (const String & filename, InputPlugin * ip,
- VFSFile && file, String && error);
 
 /* probe.cc */
 bool open_input_file (const char * filename, const char * mode,
  InputPlugin * ip, VFSFile & file, String * error = nullptr);
 InputPlugin * load_input_plugin (PluginHandle * decoder, String * error = nullptr);
 
-/* internal versions of aud_file_* functions;
- * these allow reuse of the same file handle during probing */
-PluginHandle * file_find_decoder (const char * filename, bool fast,
- VFSFile & file, String * error = nullptr);
-bool file_read_tag (const char * filename, PluginHandle * decoder,
- VFSFile & file, Tuple * tuple, Index<char> * image, String * error = nullptr);
-
 /* runtime.cc */
 extern size_t misc_bytes_allocated;
 
@@ -130,9 +121,25 @@ bool same_basename (const char * a, const char * b);
 const char * last_path_element (const char * path);
 void cut_path_element (char * path, int pos);
 
+bool is_cuesheet_entry (const char * filename);
+bool is_subtune (const char * filename);
+StringBuf strip_subtune (const char * filename);
+
 unsigned int32_hash (unsigned val);
 unsigned ptr_hash (const void * ptr);
 
+struct IntHashKey
+{
+    int val;
+
+    constexpr IntHashKey (int val) :
+        val (val) {}
+    operator int () const
+        { return val; }
+    unsigned hash () const
+        { return int32_hash (val); }
+};
+
 /* vis-runner.cc */
 void vis_runner_start_stop (bool playing, bool paused);
 void vis_runner_pass_audio (int time, const Index<float> & data, int channels, int rate);
