@@ -43,13 +43,14 @@ bool plugin_enable_secondary (PluginHandle * plugin, bool enable);
 /* plugin-load.c */
 void plugin_system_init ();
 void plugin_system_cleanup ();
-bool plugin_check_flags (int version);
+bool plugin_check_flags (int flags);
 Plugin * plugin_load (const char * path);
 
 /* plugin-registry.c */
 void plugin_registry_load ();
 void plugin_registry_prune ();
 void plugin_registry_save ();
+void plugin_registry_cleanup ();
 
 void plugin_register (const char * path, int timestamp);
 PluginEnabled plugin_get_enabled (PluginHandle * plugin);
