@@ -35,6 +35,7 @@ static const char * const audgui_defaults[] = {
     "close_dialog_add", "FALSE",
     "close_dialog_open", "TRUE",
     "close_jtf_dialog", "TRUE",
+    "record", "FALSE",
     "remember_jtf_entry", "TRUE",
     nullptr
 };
@@ -42,6 +43,7 @@ static const char * const audgui_defaults[] = {
 static const char * const window_names[AUDGUI_NUM_UNIQUE_WINDOWS] = {
     "about_win",
     "equalizer_win",
+    "eq_preset_win",
     "filebrowser_win",
     nullptr, /* infopopup position is not saved */
     "info_win",
@@ -49,6 +51,7 @@ static const char * const window_names[AUDGUI_NUM_UNIQUE_WINDOWS] = {
     "jump_to_track_win",
     "playlist_export_win",
     "playlist_import_win",
+    "preset_browser_win",
     "queue_manager_win",
     "url_opener_win"
 };
@@ -65,6 +68,9 @@ static gboolean configure_cb (GtkWidget * window, GdkEventConfigure * event, con
         gtk_window_get_position ((GtkWindow *) window, & pos[0], & pos[1]);
         gtk_window_get_size ((GtkWindow *) window, & pos[2], & pos[3]);
 
+        pos[2] = audgui_to_portable_dpi (pos[2]);
+        pos[3] = audgui_to_portable_dpi (pos[3]);
+
         aud_set_str ("audgui", name, int_array_to_str (pos, 4));
     }
 
@@ -88,6 +94,9 @@ void audgui_show_unique_window (int id, GtkWidget * widget)
 
         if (str_to_int_array (str, pos, 4))
         {
+            pos[2] = audgui_to_native_dpi (pos[2]);
+            pos[3] = audgui_to_native_dpi (pos[3]);
+
             gtk_window_move ((GtkWindow *) widget, pos[0], pos[1]);
             gtk_window_set_default_size ((GtkWindow *) widget, pos[2], pos[3]);
         }
@@ -125,7 +134,7 @@ static void playlist_set_playing_cb (void *, void *)
 
 static void playlist_position_cb (void * list, void *)
 {
-    if (aud::from_ptr<int> (list) == aud_playlist_get_playing ())
+    if (aud::from_ptr<Playlist> (list) == Playlist::playing_playlist ())
         audgui_pixbuf_uncache ();
 }
 
@@ -168,5 +177,4 @@ EXPORT void audgui_cleanup ()
 
     plugin_menu_cleanup ();
     plugin_prefs_cleanup ();
-    urilist_cleanup ();
 }
