@@ -17,11 +17,17 @@
  * the use of this software.
  */
 
+#include <math.h>
 #include <string.h>
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#define AUD_GLIB_INTEGRATION
 #include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudcore/i18n.h>
@@ -31,6 +37,43 @@
 #include "libaudgui.h"
 #include "libaudgui-gtk.h"
 
+#define PORTABLE_DPI 96
+
+EXPORT int audgui_get_dpi ()
+{
+    static int dpi = 0;
+
+    if (! dpi)
+    {
+#ifdef _WIN32
+        HDC screen = GetDC (nullptr);
+        dpi = (GetDeviceCaps (screen, LOGPIXELSX) + GetDeviceCaps (screen, LOGPIXELSY)) / 2;
+        ReleaseDC (nullptr, screen);
+#else
+        GdkScreen * screen = gdk_screen_get_default ();
+
+        /* force GTK settings to be loaded for the GDK screen */
+        (void) gtk_settings_get_for_screen (screen);
+
+        dpi = round (gdk_screen_get_resolution (screen));
+#endif
+
+        dpi = aud::max (PORTABLE_DPI, dpi);
+    }
+
+    return dpi;
+}
+
+EXPORT int audgui_to_native_dpi (int size)
+{
+    return aud::rescale (size, PORTABLE_DPI, audgui_get_dpi ());
+}
+
+EXPORT int audgui_to_portable_dpi (int size)
+{
+    return aud::rescale (size, audgui_get_dpi (), PORTABLE_DPI);
+}
+
 EXPORT int audgui_get_digit_width (GtkWidget * widget)
 {
     int width;
@@ -105,6 +148,89 @@ EXPORT GtkWidget * audgui_button_new (const char * text, const char * icon,
     return button;
 }
 
+struct FileEntryData {
+    GtkFileChooserAction action;
+    String title;
+};
+
+static void entry_response_cb (GtkWidget * dialog, int response, GtkWidget * entry)
+{
+    if (response == GTK_RESPONSE_ACCEPT)
+    {
+        CharPtr uri (gtk_file_chooser_get_uri ((GtkFileChooser *) dialog));
+        if (uri)
+            audgui_file_entry_set_uri (entry, uri);
+    }
+
+    gtk_widget_destroy (dialog);
+}
+
+static void entry_browse_cb (GtkWidget * entry, GtkEntryIconPosition pos,
+ GdkEvent * event, const FileEntryData * data)
+{
+    GtkWidget * dialog = gtk_file_chooser_dialog_new (data->title, nullptr,
+     data->action, _("Open"), GTK_RESPONSE_ACCEPT, _("Cancel"),
+     GTK_RESPONSE_REJECT, nullptr);
+
+    gtk_file_chooser_set_local_only ((GtkFileChooser *) dialog, false);
+
+    String uri = audgui_file_entry_get_uri (entry);
+    if (uri)
+        gtk_file_chooser_set_uri ((GtkFileChooser *) dialog, uri);
+
+    g_signal_connect (dialog, "response", (GCallback) entry_response_cb, entry);
+    g_signal_connect_object (entry, "destroy", (GCallback) gtk_widget_destroy,
+     dialog, G_CONNECT_SWAPPED);
+
+    gtk_widget_show (dialog);
+}
+
+EXPORT GtkWidget * audgui_file_entry_new (GtkFileChooserAction action, const char * title)
+{
+    GtkWidget * entry = gtk_entry_new ();
+
+    auto data = new FileEntryData {action, String (title)};
+    auto destroy_cb = [] (void * data) { delete (FileEntryData *) data; };
+    g_object_set_data_full ((GObject *) entry, "file-entry-data", data, destroy_cb);
+
+    gtk_entry_set_icon_from_icon_name ((GtkEntry *) entry,
+     GTK_ENTRY_ICON_SECONDARY, "document-open");
+    g_signal_connect (entry, "icon-press", (GCallback) entry_browse_cb, data);
+
+    return entry;
+}
+
+EXPORT String audgui_file_entry_get_uri (GtkWidget * entry)
+{
+    const char * text = gtk_entry_get_text ((GtkEntry *) entry);
+
+    if (! text[0])
+        return String ();
+    else if (strstr (text, "://"))
+        return String (text);
+    else
+        return String (filename_to_uri (filename_normalize (filename_expand (str_copy (text)))));
+}
+
+EXPORT void audgui_file_entry_set_uri (GtkWidget * entry, const char * uri)
+{
+    if (! uri || ! uri[0])
+    {
+        gtk_entry_set_text ((GtkEntry *) entry, "");
+        return;
+    }
+
+    StringBuf path = uri_to_filename (uri, false);
+    gtk_entry_set_text ((GtkEntry *) entry, path ? filename_contract (std::move (path)) : uri);
+    gtk_editable_set_position ((GtkEditable *) entry, -1);
+}
+
+static void set_label_wrap (GtkWidget * label, void *)
+{
+    if (GTK_IS_LABEL (label))
+        gtk_label_set_line_wrap_mode ((GtkLabel *) label, PANGO_WRAP_WORD_CHAR);
+}
+
 EXPORT GtkWidget * audgui_dialog_new (GtkMessageType type, const char * title,
  const char * text, GtkWidget * button1, GtkWidget * button2)
 {
@@ -112,6 +238,9 @@ EXPORT GtkWidget * audgui_dialog_new (GtkMessageType type, const char * title,
      GTK_BUTTONS_NONE, "%s", text);
     gtk_window_set_title ((GtkWindow *) dialog, title);
 
+    GtkWidget * box = gtk_message_dialog_get_message_area ((GtkMessageDialog *) dialog);
+    gtk_container_foreach ((GtkContainer *) box, set_label_wrap, nullptr);
+
     if (button2)
     {
         gtk_dialog_add_action_widget ((GtkDialog *) dialog, button2, GTK_RESPONSE_NONE);
@@ -145,7 +274,7 @@ EXPORT void audgui_simple_message (GtkWidget * * widget, GtkMessageType type,
 
     if (* widget)
     {
-        const char * old = nullptr;
+        char * old = nullptr;
         g_object_get ((GObject *) * widget, "text", & old, nullptr);
         g_return_if_fail (old);
 
@@ -160,6 +289,7 @@ EXPORT void audgui_simple_message (GtkWidget * * widget, GtkMessageType type,
             g_object_set_data ((GObject *) * widget, "messages", GINT_TO_POINTER (messages + 1));
         }
 
+        g_free (old);
         gtk_window_present ((GtkWindow *) * widget);
     }
     else
