@@ -18,46 +18,33 @@
  * the use of this software.
  */
 
-#include <QAbstractTableModel>
-#include <QTreeView>
+#ifndef LIBAUDQT_INFO_WIDGET_H
+#define LIBAUDQT_INFO_WIDGET_H
 
-#include <libaudcore/tuple.h>
+#include <QTreeView>
+#include <libaudqt/export.h>
 
 class PluginHandle;
+class Tuple;
 
 namespace audqt {
 
-class InfoModel : public QAbstractTableModel {
-public:
-    InfoModel (QObject * parent = nullptr);
+class InfoModel;
 
-    int rowCount (const QModelIndex & parent = QModelIndex()) const;
-    int columnCount (const QModelIndex & parent = QModelIndex()) const;
-    QVariant data (const QModelIndex & index, int role = Qt::DisplayRole) const;
-    bool setData (const QModelIndex & index, const QVariant & value, int role = Qt::EditRole);
-    Qt::ItemFlags flags (const QModelIndex & index) const;
-
-    void setTupleData (const Tuple & tuple, String filename, PluginHandle * plugin);
-    bool updateFile () const;
-
-private:
-    Tuple m_tuple;
-    String m_filename;
-    PluginHandle * m_plugin;
-    bool m_dirty;
-};
-
-class InfoWidget : public QTreeView {
+class LIBAUDQT_PUBLIC InfoWidget : public QTreeView
+{
 public:
     InfoWidget (QWidget * parent = nullptr);
     ~InfoWidget ();
 
-    void fillInfo (int playlist, int entry, const char * filename, const Tuple & tuple,
+    void fillInfo (const char * filename, const Tuple & tuple,
      PluginHandle * decoder, bool updating_enabled);
     bool updateFile ();
 
 private:
-    InfoModel m_model;
+    InfoModel * m_model;
 };
 
 } // namespace audqt
+
+#endif // LIBAUDQT_INFO_WIDGET_H
