@@ -44,7 +44,7 @@ class InfoWindow : public QDialog
 public:
     InfoWindow (QWidget * parent = nullptr);
 
-    void fillInfo (int playlist, int entry, const char * filename, const Tuple & tuple,
+    void fillInfo (const char * filename, const Tuple & tuple,
      PluginHandle * decoder, bool updating_enabled);
 
 private:
@@ -61,12 +61,13 @@ private:
 InfoWindow::InfoWindow (QWidget * parent) : QDialog (parent)
 {
     setWindowTitle (_("Song Info"));
+    setContentsMargins (margins.TwoPt);
 
-    auto hbox = new QHBoxLayout;
+    auto hbox = make_hbox (nullptr);
     hbox->addWidget (& m_image);
     hbox->addWidget (& m_infowidget);
 
-    auto vbox = new QVBoxLayout (this);
+    auto vbox = make_vbox (this);
     vbox->addLayout (hbox);
 
     auto bbox = new QDialogButtonBox (QDialogButtonBox::Save | QDialogButtonBox::Close, this);
@@ -82,23 +83,24 @@ InfoWindow::InfoWindow (QWidget * parent) : QDialog (parent)
     QObject::connect (bbox, & QDialogButtonBox::rejected, this, & QObject::deleteLater);
 }
 
-void InfoWindow::fillInfo (int playlist, int entry, const char * filename, const Tuple & tuple,
+void InfoWindow::fillInfo (const char * filename, const Tuple & tuple,
  PluginHandle * decoder, bool updating_enabled)
 {
     m_filename = String (filename);
     displayImage (filename);
-    m_infowidget.fillInfo (playlist, entry, filename, tuple, decoder, updating_enabled);
+    m_infowidget.fillInfo (filename, tuple, decoder, updating_enabled);
 }
 
 void InfoWindow::displayImage (const char * filename)
 {
     if (! strcmp_safe (filename, m_filename))
-        m_image.setPixmap (art_request (filename));
+        m_image.setPixmap (art_request (filename, 2 * sizes.OneInch, 2 * sizes.OneInch));
 }
 
 static InfoWindow * s_infowin = nullptr;
 
-EXPORT void infowin_show (int playlist, int entry)
+static void show_infowin (const char * filename,
+ const Tuple & tuple, PluginHandle * decoder, bool can_write)
 {
     if (! s_infowin)
     {
@@ -110,42 +112,46 @@ EXPORT void infowin_show (int playlist, int entry)
         });
     }
 
-    String filename = aud_playlist_entry_get_filename (playlist, entry);
+    s_infowin->fillInfo (filename, tuple, decoder, can_write);
+    s_infowin->resize (6 * sizes.OneInch, 3 * sizes.OneInch);
+    window_bring_to_front (s_infowin);
+}
+
+EXPORT void infowin_show (Playlist playlist, int entry)
+{
+    String filename = playlist.entry_filename (entry);
     if (! filename)
         return;
 
-    PluginHandle * decoder = aud_playlist_entry_get_decoder (playlist, entry);
-    if (! decoder)
-        return;
+    String error;
+    PluginHandle * decoder = playlist.entry_decoder (entry, Playlist::Wait, & error);
+    Tuple tuple = decoder ? playlist.entry_tuple (entry, Playlist::Wait, & error) : Tuple ();
 
-    Tuple tuple = aud_playlist_entry_get_tuple (playlist, entry);
-
-    if (tuple)
+    if (decoder && tuple.valid () && ! aud_custom_infowin (filename, decoder))
     {
+        /* cuesheet entries cannot be updated */
+        bool can_write = aud_file_can_write_tuple (filename, decoder) &&
+         ! tuple.is_set (Tuple::StartTime);
+
         tuple.delete_fallbacks ();
-        s_infowin->fillInfo (playlist, entry, filename, tuple, decoder,
-            aud_file_can_write_tuple (filename, decoder));
+        show_infowin (filename, tuple, decoder, can_write);
     }
     else
-        aud_ui_show_error (str_printf (_("No info available for %s.\n"),
-            (const char *) filename));
+        infowin_hide ();
 
-    s_infowin->resize (700, 300);
-
-    window_bring_to_front (s_infowin);
+    if (error)
+        aud_ui_show_error (str_printf (_("Error opening %s:\n%s"),
+         (const char *) filename, (const char *) error));
 }
 
 EXPORT void infowin_show_current ()
 {
-    int playlist = aud_playlist_get_playing ();
-    int position;
+    auto playlist = Playlist::playing_playlist ();
+    if (playlist == Playlist ())
+        playlist = Playlist::active_playlist ();
 
-    if (playlist == -1)
-        playlist = aud_playlist_get_active ();
-
-    position = aud_playlist_get_position (playlist);
-
-    if (position == -1)
+    int position = playlist.get_position ();
+    if (position < 0)
         return;
 
     infowin_show (playlist, position);
