@@ -19,6 +19,7 @@
  */
 
 #include <QAction>
+#include <QCheckBox>
 #include <QComboBox>
 #include <QDialog>
 #include <QDialogButtonBox>
@@ -55,13 +56,63 @@
 
 namespace audqt {
 
-struct Category {
-    const char * icon_path;
-    const char * name;
+class PrefsWindow : public QDialog
+{
+public:
+    static PrefsWindow * get_instance () {
+        if (! instance)
+            (void) new PrefsWindow;
+        return instance;
+    }
+
+    static void destroy_instance () {
+        if (instance)
+            delete instance;
+    }
+
+    static ArrayRef<ComboItem> get_output_combo () {
+        return {instance->output_combo_elements.begin (),
+                instance->output_combo_elements.len ()};
+    }
+
+    static int output_combo_selected;
+    static void output_combo_changed () { instance->output_change (); }
+
+    static void * get_output_config_button () { return instance->output_config_button; }
+    static void * get_output_about_button () { return instance->output_about_button; }
+
+    static void * get_record_checkbox () { return instance->record_checkbox; }
+    static void * get_record_config_button () { return instance->record_config_button; }
+    static void * get_record_about_button () { return instance->record_about_button; }
+
+private:
+    static PrefsWindow * instance;
+
+    PrefsWindow ();
+    ~PrefsWindow () { instance = nullptr; }
+
+    Index<ComboItem> output_combo_elements;
+    QPushButton * output_config_button, * output_about_button;
+
+    QCheckBox * record_checkbox;
+    QPushButton * record_config_button, * record_about_button;
+
+    void output_setup ();
+    void output_change ();
+
+    void record_setup ();
+    void record_update ();
+
+    const HookReceiver<PrefsWindow>
+     record_hook {"enable record", this, & PrefsWindow::record_update};
 };
 
-struct PluginCategory {
-    PluginType type;
+/* static data */
+PrefsWindow * PrefsWindow::instance = nullptr;
+int PrefsWindow::output_combo_selected;
+
+struct Category {
+    const char * icon_path;
     const char * name;
 };
 
@@ -89,28 +140,20 @@ static const Category categories[] = {
     { "plugins.png", N_("Plugins") }
 };
 
-static const PluginCategory plugin_categories[] = {
-    { PluginType::General, N_("General") },
-    { PluginType::Effect, N_("Effect") },
-    { PluginType::Vis, N_("Visualization") },
-    { PluginType::Input, N_("Input") },
-    { PluginType::Playlist, N_("Playlist") },
-    { PluginType::Transport, N_("Transport") }
-};
-
 static const TitleFieldTag title_field_tags[] = {
-    { N_("Artist")     , "${artist}" },
-    { N_("Album")      , "${album}" },
-    { N_("Title")      , "${title}" },
+    { N_("Artist")      , "${artist}" },
+    { N_("Album")       , "${album}" },
+    { N_("Album Artist"), "${album-artist}" },
+    { N_("Title")       , "${title}" },
     { N_("Track number"), "${track-number}" },
-    { N_("Genre")      , "${genre}" },
-    { N_("File name")  , "${file-name}" },
-    { N_("File path")  , "${file-path}" },
-    { N_("Date")       , "${date}" },
-    { N_("Year")       , "${year}" },
-    { N_("Comment")    , "${comment}" },
-    { N_("Codec")      , "${codec}" },
-    { N_("Quality")    , "${quality}" }
+    { N_("Genre")       , "${genre}" },
+    { N_("File name")   , "${file-name}" },
+    { N_("File path")   , "${file-path}" },
+    { N_("Date")        , "${date}" },
+    { N_("Year")        , "${year}" },
+    { N_("Comment")     , "${comment}" },
+    { N_("Codec")       , "${codec}" },
+    { N_("Quality")     , "${quality}" }
 };
 
 static const ComboItem chardet_detector_presets[] = {
@@ -136,6 +179,19 @@ static const ComboItem bitdepth_elements[] = {
     ComboItem (N_("Floating point"), 0)
 };
 
+static const ComboItem record_elements[] = {
+    ComboItem (N_("As decoded"), (int) OutputStream::AsDecoded),
+    ComboItem (N_("After applying ReplayGain"), (int) OutputStream::AfterReplayGain),
+    ComboItem (N_("After applying effects"), (int) OutputStream::AfterEffects),
+    ComboItem (N_("After applying equalization"), (int) OutputStream::AfterEqualizer)
+};
+
+static const ComboItem replaygainmode_elements[] = {
+    ComboItem (N_("Track"), (int) ReplayGainMode::Track),
+    ComboItem (N_("Album"), (int) ReplayGainMode::Album),
+    ComboItem (N_("Based on shuffle"), (int) ReplayGainMode::Automatic)
+};
+
 static Index<ComboItem> iface_combo_elements;
 static int iface_combo_selected;
 static QWidget * iface_prefs_box;
@@ -145,41 +201,37 @@ static void iface_combo_changed ();
 static void * iface_create_prefs_box ();
 
 static const PreferencesWidget appearance_page_widgets[] = {
-    WidgetLabel (N_("<b>Interface Settings</b>")),
-    WidgetCombo (N_("Interface plugin:"),
+    WidgetCombo (N_("Interface:"),
         WidgetInt (iface_combo_selected, iface_combo_changed),
         {0, iface_combo_fill}),
+    WidgetSeparator ({true}),
     WidgetCustomQt (iface_create_prefs_box)
 };
 
-static Index<ComboItem> output_combo_elements;
-static int output_combo_selected;
-static QPushButton * output_config_button;
-static QPushButton * output_about_button;
-
-static ArrayRef<ComboItem> output_combo_fill ();
-static void output_combo_changed ();
-static void * output_create_config_button ();
-static void * output_create_about_button ();
 static void output_bit_depth_changed ();
 
 static const PreferencesWidget output_combo_widgets[] = {
     WidgetCombo (N_("Output plugin:"),
-        WidgetInt (output_combo_selected, output_combo_changed, "audqt update output combo"),
-        {0, output_combo_fill}),
-    WidgetCustomQt (output_create_config_button),
-    WidgetCustomQt (output_create_about_button)
+        WidgetInt (PrefsWindow::output_combo_selected,
+                   PrefsWindow::output_combo_changed,
+                   "audqt update output combo"),
+        {0, PrefsWindow::get_output_combo}),
+    WidgetCustomQt (PrefsWindow::get_output_config_button),
+    WidgetCustomQt (PrefsWindow::get_output_about_button)
+};
+
+static const PreferencesWidget record_buttons[] = {
+    WidgetCustomQt (PrefsWindow::get_record_config_button),
+    WidgetCustomQt (PrefsWindow::get_record_about_button)
 };
 
 static const PreferencesWidget gain_table[] = {
     WidgetSpin (N_("Amplify all files:"),
         WidgetFloat (0, "replay_gain_preamp"),
-        {-15, 15, 0.1, N_("dB")},
-        WIDGET_CHILD),
+        {-15, 15, 0.1, N_("dB")}),
     WidgetSpin (N_("Amplify untagged files:"),
         WidgetFloat (0, "default_gain"),
-        {-15, 15, 0.1, N_("dB")},
-        WIDGET_CHILD)
+        {-15, 15, 0.1, N_("dB")})
 };
 
 static const PreferencesWidget audio_page_widgets[] = {
@@ -195,17 +247,23 @@ static const PreferencesWidget audio_page_widgets[] = {
         WidgetBool (0, "soft_clipping")),
     WidgetCheck (N_("Use software volume control (not recommended)"),
         WidgetBool (0, "software_volume_control")),
+    WidgetLabel (N_("<b>Recording Settings</b>")),
+    WidgetCustomQt (PrefsWindow::get_record_checkbox),
+    WidgetBox ({{record_buttons}, true},
+        WIDGET_CHILD),
+    WidgetCombo (N_("Record stream:"),
+        WidgetInt (0, "record_stream"),
+        {{record_elements}}),
     WidgetLabel (N_("<b>ReplayGain</b>")),
     WidgetCheck (N_("Enable ReplayGain"),
         WidgetBool (0, "enable_replay_gain")),
-    WidgetCheck (N_("Album mode"),
-        WidgetBool (0, "replay_gain_album"),
+    WidgetCombo (N_("Mode:"),
+        WidgetInt (0, "replay_gain_mode"),
+        {{replaygainmode_elements}},
         WIDGET_CHILD),
     WidgetCheck (N_("Prevent clipping (recommended)"),
         WidgetBool (0, "enable_clipping_prevention"),
         WIDGET_CHILD),
-    WidgetLabel (N_("<b>Adjust Levels</b>"),
-        WIDGET_CHILD),
     WidgetTable ({{gain_table}},
         WIDGET_CHILD)
 };
@@ -249,7 +307,6 @@ static const PreferencesWidget chardet_elements[] = {
         WidgetString (0, "chardet_fallback"))
 };
 
-
 static void send_title_change ();
 static void * create_titlestring_table ();
 
@@ -266,8 +323,6 @@ static const PreferencesWidget playlist_page_widgets[] = {
         WidgetBool (0, "clear_playlist")),
     WidgetCheck (N_("Open files in a temporary playlist"),
         WidgetBool (0, "open_to_temporary")),
-    WidgetCheck (N_("Do not load metadata for songs until played"),
-        WidgetBool (0, "metadata_on_play")),
     WidgetLabel (N_("<b>Song Display</b>")),
     WidgetCheck (N_("Show song numbers"),
         WidgetBool (0, "show_numbers_in_pl", send_title_change)),
@@ -343,6 +398,8 @@ static void * create_titlestring_table ()
 {
     QWidget * w = new QWidget;
     QGridLayout * l = new QGridLayout (w);
+    l->setContentsMargins (0, 0, 0, 0);
+    l->setSpacing (sizes.TwoPt);
 
     QLabel * lbl = new QLabel (_("Title format:"), w);
     l->addWidget (lbl, 0, 0);
@@ -369,13 +426,12 @@ static void * create_titlestring_table ()
             cbox->setCurrentIndex (i);
     }
 
-    QObject::connect (le, &QLineEdit::textChanged, [=] (const QString & text) {
+    QObject::connect (le, & QLineEdit::textChanged, [] (const QString & text) {
         aud_set_str (nullptr, "generic_title_format", text.toUtf8 ().data ());
     });
 
-    QObject::connect (cbox,
-                      static_cast <void (QComboBox::*) (int)> (&QComboBox::currentIndexChanged),
-                      [=] (int idx) {
+    void (QComboBox::* signal) (int) = & QComboBox::currentIndexChanged;
+    QObject::connect (cbox, signal, [le] (int idx) {
         if (idx < TITLESTRING_NPRESETS)
             le->setText (titlestring_presets [idx]);
     });
@@ -384,7 +440,6 @@ static void * create_titlestring_table ()
     QPushButton * btn_mnu = new QPushButton (w);
     btn_mnu->setFixedWidth (btn_mnu->sizeHint ().height ());
     btn_mnu->setIcon (QIcon::fromTheme ("list-add"));
-    btn_mnu->setIconSize (QSize (16, 16));
     l->addWidget (btn_mnu, 1, 2);
 
     QMenu * mnu_fields = new QMenu (w);
@@ -426,10 +481,7 @@ static void iface_fill_prefs_box ()
     Plugin * header = (Plugin *) aud_plugin_get_header (aud_plugin_get_current (PluginType::Iface));
     if (header && header->info.prefs)
     {
-        QVBoxLayout * vbox = new QVBoxLayout (iface_prefs_box);
-
-        vbox->setContentsMargins (0, 0, 0, 0);
-        vbox->setSpacing (4);
+        auto vbox = make_vbox (iface_prefs_box, sizes.TwoPt);
         prefs_populate (vbox, header->info.prefs->widgets, header->info.domain);
     }
 }
@@ -456,8 +508,8 @@ static ArrayRef<ComboItem> iface_combo_fill ()
     if (! iface_combo_elements.len ())
     {
         iface_combo_elements = fill_plugin_combo (PluginType::Iface);
-        iface_combo_selected = aud_plugin_list (PluginType::Iface)
-         .find (aud_plugin_get_current (PluginType::Iface));
+        iface_combo_selected = aud_plugin_list (PluginType::Iface).
+         find (aud_plugin_get_current (PluginType::Iface));
     }
 
     return {iface_combo_elements.begin (), iface_combo_elements.len ()};
@@ -470,64 +522,6 @@ static void * iface_create_prefs_box ()
     return iface_prefs_box;
 }
 
-static void output_combo_changed ()
-{
-    auto & list = aud_plugin_list (PluginType::Output);
-    PluginHandle * plugin = list[output_combo_selected];
-
-    if (aud_plugin_enable (plugin, true))
-    {
-        output_config_button->setEnabled (aud_plugin_has_configure (plugin));
-        output_about_button->setEnabled (aud_plugin_has_about (plugin));
-    }
-    else
-    {
-        /* set combo box back to current output */
-        output_combo_selected = list.find (aud_plugin_get_current (PluginType::Output));
-        hook_call ("audqt update output combo", nullptr);
-    }
-}
-
-static void * output_create_config_button ()
-{
-    bool enabled = aud_plugin_has_configure (aud_plugin_get_current (PluginType::Output));
-
-    output_config_button = new QPushButton (translate_str (N_("_Settings")));
-    output_config_button->setEnabled (enabled);
-
-    QObject::connect (output_config_button, & QAbstractButton::clicked, [=] (bool) {
-        plugin_prefs (aud_plugin_get_current (PluginType::Output));
-    });
-
-    return output_config_button;
-}
-
-static void * output_create_about_button ()
-{
-    bool enabled = aud_plugin_has_about (aud_plugin_get_current (PluginType::Output));
-
-    output_about_button = new QPushButton (translate_str (N_("_About")));
-    output_about_button->setEnabled (enabled);
-
-    QObject::connect (output_about_button, &QAbstractButton::clicked, [=] (bool) {
-        plugin_about (aud_plugin_get_current (PluginType::Output));
-    });
-
-    return output_about_button;
-}
-
-static ArrayRef<ComboItem> output_combo_fill ()
-{
-    if (! output_combo_elements.len ())
-    {
-        output_combo_elements = fill_plugin_combo (PluginType::Output);
-        output_combo_selected = aud_plugin_list (PluginType::Output)
-         .find (aud_plugin_get_current (PluginType::Output));
-    }
-
-    return {output_combo_elements.begin (), output_combo_elements.len ()};
-}
-
 static void output_bit_depth_changed ()
 {
     aud_output_reset (OutputReset::ReopenStream);
@@ -536,91 +530,81 @@ static void output_bit_depth_changed ()
 static void create_category (QStackedWidget * notebook, ArrayRef<PreferencesWidget> widgets)
 {
     QWidget * w = new QWidget;
-    QVBoxLayout * vbox = new QVBoxLayout (w);
-
-    vbox->setContentsMargins (0, 0, 0, 0);
-    vbox->setSpacing (4);
+    auto vbox = make_vbox (w, sizes.TwoPt);
     prefs_populate (vbox, widgets, nullptr);
     vbox->addStretch (1);
 
     notebook->addWidget (w);
 }
 
-static void create_plugin_category_page (PluginType category_id, const char * category_name, QTabWidget * parent)
-{
-    QTreeView * view = new QTreeView;
-    QHeaderView * header = view->header ();
+static QTreeView * s_plugin_view;
+static PluginListModel * s_plugin_model;
 
-    view->setIndentation (0);
-    view->setModel (new PluginListModel (view, category_id));
-    view->setSelectionMode (view->NoSelection);
+static void create_plugin_category (QStackedWidget * parent)
+{
+    s_plugin_view = new QTreeView (parent);
+    s_plugin_model = new PluginListModel (s_plugin_view);
+
+    s_plugin_view->setModel (s_plugin_model);
+    s_plugin_view->setSelectionMode (QTreeView::NoSelection);
+
+    auto header = s_plugin_view->header ();
 
     header->hide ();
     header->setSectionResizeMode (header->ResizeToContents);
     header->setStretchLastSection (false);
 
-    parent->addTab (view, category_name);
+    parent->addWidget (s_plugin_view);
 
-    QObject::connect (view, & QAbstractItemView::clicked,
-     [category_id] (const QModelIndex & index)
+    QObject::connect (s_plugin_view, & QAbstractItemView::clicked, [] (const QModelIndex & index)
     {
-        int row = index.row ();
-        auto & list = aud_plugin_list (category_id);
-
-        if (row < 0 || row >= list.len () || ! aud_plugin_get_enabled (list[row]))
+        auto p = s_plugin_model->pluginForIndex (index);
+        if (! p)
             return;
 
         switch (index.column ())
         {
         case PluginListModel::AboutColumn:
-            plugin_about (list[row]);
+            plugin_about (p);
             break;
         case PluginListModel::SettingsColumn:
-            plugin_prefs (list[row]);
+            plugin_prefs (p);
             break;
         }
     });
 }
 
-static QTabWidget * plugin_tabs = nullptr;
-
-static void create_plugin_category (QStackedWidget * parent)
-{
-    plugin_tabs = new QTabWidget;
-
-    for (const PluginCategory & w : plugin_categories)
-    {
-        create_plugin_category_page (w.type, _(w.name), plugin_tabs);
-    }
-
-    parent->addWidget (plugin_tabs);
-}
-
-static QDialog * s_prefswin = nullptr;
 static QStackedWidget * s_category_notebook = nullptr;
 
-static void create_prefs_window ()
+PrefsWindow::PrefsWindow () :
+    output_combo_elements (fill_plugin_combo (PluginType::Output)),
+    output_config_button (new QPushButton (translate_str (N_("_Settings")))),
+    output_about_button (new QPushButton (translate_str (N_("_About")))),
+    record_checkbox (new QCheckBox),
+    record_config_button (new QPushButton (translate_str (N_("_Settings")))),
+    record_about_button (new QPushButton (translate_str (N_("_About"))))
 {
-    s_prefswin = new QDialog;
-    s_prefswin->setWindowTitle (_("Audacious Settings"));
-    s_prefswin->setAttribute (Qt::WA_DeleteOnClose);
+    /* initialize static data */
+    instance = this;
+    output_combo_selected = aud_plugin_list (PluginType::Output)
+     .find (aud_plugin_get_current (PluginType::Output));
 
-    QObject::connect (s_prefswin, & QObject::destroyed, [] () {
-        s_prefswin = nullptr;
-    });
-
-    QVBoxLayout * vbox_parent = new QVBoxLayout (s_prefswin);
-    vbox_parent->setSpacing (0);
-    vbox_parent->setContentsMargins (0, 0, 0, 0);
+    setAttribute (Qt::WA_DeleteOnClose);
+    setWindowTitle (_("Audacious Settings"));
+    setContentsMargins (0, 0, 0, 0);
 
     QToolBar * toolbar = new QToolBar;
     toolbar->setToolButtonStyle (Qt::ToolButtonTextUnderIcon);
-    vbox_parent->addWidget (toolbar);
 
     QWidget * child = new QWidget;
-    QVBoxLayout * child_vbox = new QVBoxLayout (child);
+    child->setContentsMargins (margins.FourPt);
+
+    auto vbox_parent = make_vbox (this);
+    vbox_parent->addWidget (toolbar);
     vbox_parent->addWidget (child);
 
+    auto child_vbox = make_vbox (child);
+
     s_category_notebook = new QStackedWidget;
     child_vbox->addWidget (s_category_notebook);
 
@@ -635,9 +619,9 @@ static void create_prefs_window ()
     bbox->button (QDialogButtonBox::Close)->setText (translate_str (N_("_Close")));
     child_vbox->addWidget (bbox);
 
-    QObject::connect (bbox, & QDialogButtonBox::rejected, s_prefswin, & QObject::deleteLater);
+    QObject::connect (bbox, & QDialogButtonBox::rejected, this, & QObject::deleteLater);
 
-    QSignalMapper * mapper = new QSignalMapper (s_prefswin);
+    QSignalMapper * mapper = new QSignalMapper (this);
     const char * data_dir = aud_get_path (AudPath::DataDir);
 
     QObject::connect (mapper, static_cast <void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
@@ -649,24 +633,101 @@ static void create_prefs_window ()
         QAction * a = new QAction (ico, translate_str (categories[i].name), toolbar);
 
         toolbar->addAction (a);
-
         mapper->setMapping (a, i);
 
-        QObject::connect (a, & QAction::triggered, mapper, static_cast <void (QSignalMapper::*)()>(& QSignalMapper::map));
+        void (QSignalMapper::* slot) () = & QSignalMapper::map;
+        QObject::connect (a, & QAction::triggered, mapper, slot);
+    }
+
+    output_setup ();
+    record_setup ();
+    record_update ();
+}
+
+void PrefsWindow::output_setup ()
+{
+    auto p = aud_plugin_get_current (PluginType::Output);
+
+    output_config_button->setEnabled (aud_plugin_has_configure (p));
+    output_about_button->setEnabled (aud_plugin_has_about (p));
+
+    QObject::connect (output_config_button, & QPushButton::clicked, [] (bool) {
+        plugin_prefs (aud_plugin_get_current (PluginType::Output));
+    });
+
+    QObject::connect (output_about_button, & QPushButton::clicked, [] (bool) {
+        plugin_about (aud_plugin_get_current (PluginType::Output));
+    });
+}
+
+void PrefsWindow::output_change ()
+{
+    auto & list = aud_plugin_list (PluginType::Output);
+    auto p = list[output_combo_selected];
+
+    if (aud_plugin_enable (p, true))
+    {
+        output_config_button->setEnabled (aud_plugin_has_configure (p));
+        output_about_button->setEnabled (aud_plugin_has_about (p));
+    }
+    else
+    {
+        /* set combo box back to current output */
+        output_combo_selected = list.find (aud_plugin_get_current (PluginType::Output));
+        hook_call ("audqt update output combo", nullptr);
+    }
+}
+
+void PrefsWindow::record_setup ()
+{
+    QObject::connect (record_checkbox, & QCheckBox::clicked, [] (bool checked) {
+        aud_drct_enable_record (checked);
+    });
+
+    QObject::connect (record_config_button, & QPushButton::clicked, [] (bool) {
+        if (aud_drct_get_record_enabled ())
+            plugin_prefs (aud_drct_get_record_plugin ());
+    });
+
+    QObject::connect (record_about_button, & QPushButton::clicked, [] (bool) {
+        if (aud_drct_get_record_enabled ())
+            plugin_about (aud_drct_get_record_plugin ());
+    });
+}
+
+void PrefsWindow::record_update ()
+{
+    auto p = aud_drct_get_record_plugin ();
+
+    if (p)
+    {
+        bool enabled = aud_drct_get_record_enabled ();
+        auto text = str_printf (_("Enable audio stream recording with %s"), aud_plugin_get_name (p));
+
+        record_checkbox->setEnabled (true);
+        record_checkbox->setText ((const char *) text);
+        record_checkbox->setChecked (enabled);
+        record_config_button->setEnabled (enabled && aud_plugin_has_configure (p));
+        record_about_button->setEnabled (enabled && aud_plugin_has_about (p));
+    }
+    else
+    {
+        record_checkbox->setEnabled (false);
+        record_checkbox->setText (_("No audio recording plugin available"));
+        record_checkbox->setChecked (false);
+        record_config_button->setEnabled (false);
+        record_about_button->setEnabled (false);
     }
 }
 
 EXPORT void prefswin_show ()
 {
-    if (! s_prefswin)
-        create_prefs_window ();
-
-    window_bring_to_front (s_prefswin);
+    window_bring_to_front (PrefsWindow::get_instance ());
 }
 
 EXPORT void prefswin_hide ()
 {
-    delete s_prefswin;
+    PrefsWindow::destroy_instance ();
 }
 
 EXPORT void prefswin_show_page (int id, bool show)
@@ -674,35 +735,34 @@ EXPORT void prefswin_show_page (int id, bool show)
     if (id < 0 || id > CATEGORY_COUNT)
         return;
 
-    if (! s_prefswin)
-        create_prefs_window ();
-
+    auto win = PrefsWindow::get_instance ();
     s_category_notebook->setCurrentIndex (id);
 
     if (show)
-        window_bring_to_front (s_prefswin);
+        window_bring_to_front (win);
 }
 
 EXPORT void prefswin_show_plugin_page (PluginType type)
 {
-    if (! s_prefswin)
-        create_prefs_window ();
-
     if (type == PluginType::Iface)
-        return prefswin_show_page (CATEGORY_APPEARANCE);
+        prefswin_show_page (CATEGORY_APPEARANCE);
     else if (type == PluginType::Output)
-        return prefswin_show_page (CATEGORY_AUDIO);
+        prefswin_show_page (CATEGORY_AUDIO);
     else
     {
         prefswin_show_page (CATEGORY_PLUGINS, false);
 
-        for (const PluginCategory & category : plugin_categories)
+        s_plugin_view->collapseAll ();
+
+        auto index = s_plugin_model->indexForType (type);
+        if (index.isValid ())
         {
-            if (category.type == type)
-                plugin_tabs->setCurrentIndex (& category - plugin_categories);
+            s_plugin_view->expand (index);
+            s_plugin_view->scrollTo (index, QTreeView::PositionAtTop);
+            s_plugin_view->setCurrentIndex (index);
         }
 
-        window_bring_to_front (s_prefswin);
+        window_bring_to_front (PrefsWindow::get_instance ());
     }
 }
 
