@@ -20,6 +20,7 @@
 #include <stdlib.h>
 
 #include <QApplication>
+#include <QDesktopWidget>
 #include <QPushButton>
 #include <QVBoxLayout>
 
@@ -27,6 +28,7 @@
 #include <libaudcore/i18n.h>
 #include <libaudcore/runtime.h>
 
+#include "libaudqt-internal.h"
 #include "libaudqt.h"
 
 namespace audqt {
@@ -34,6 +36,12 @@ namespace audqt {
 static int init_count;
 static QApplication * qapp;
 
+static PixelSizes sizes_local;
+static PixelMargins margins_local;
+
+EXPORT const PixelSizes & sizes = sizes_local;
+EXPORT const PixelMargins & margins = margins_local;
+
 EXPORT void init ()
 {
     if (init_count ++ || qapp)
@@ -47,8 +55,24 @@ EXPORT void init ()
     atexit ([] () { delete qapp; });
 
     qapp->setAttribute (Qt::AA_UseHighDpiPixmaps);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 3, 0)
+    qapp->setAttribute (Qt::AA_ForceRasterWidgets);
+#endif
+
     qapp->setApplicationName (_("Audacious"));
     qapp->setWindowIcon (QIcon::fromTheme (app_name));
+
+    auto desktop = qapp->desktop ();
+    sizes_local.OneInch = aud::max (96, (desktop->logicalDpiX () + desktop->logicalDpiY ()) / 2);
+    sizes_local.TwoPt = aud::rescale (2, 72, sizes_local.OneInch);
+    sizes_local.FourPt = aud::rescale (4, 72, sizes_local.OneInch);
+    sizes_local.EightPt = aud::rescale (8, 72, sizes_local.OneInch);
+
+    margins_local.TwoPt = QMargins (sizes.TwoPt, sizes.TwoPt, sizes.TwoPt, sizes.TwoPt);
+    margins_local.FourPt = QMargins (sizes.FourPt, sizes.FourPt, sizes.FourPt, sizes.FourPt);
+    margins_local.EightPt = QMargins (sizes.EightPt, sizes.EightPt, sizes.EightPt, sizes.EightPt);
+
+    log_init ();
 }
 
 EXPORT void run ()
@@ -72,6 +96,24 @@ EXPORT void cleanup ()
     log_inspector_hide ();
     prefswin_hide ();
     queue_manager_hide ();
+
+    log_cleanup ();
+}
+
+EXPORT QHBoxLayout * make_hbox (QWidget * parent, int spacing)
+{
+    auto layout = new QHBoxLayout (parent);
+    layout->setContentsMargins (0, 0, 0, 0);
+    layout->setSpacing (spacing);
+    return layout;
+}
+
+EXPORT QVBoxLayout * make_vbox (QWidget * parent, int spacing)
+{
+    auto layout = new QVBoxLayout (parent);
+    layout->setContentsMargins (0, 0, 0, 0);
+    layout->setSpacing (spacing);
+    return layout;
 }
 
 EXPORT void enable_layout (QLayout * layout, bool enabled)
@@ -131,6 +173,10 @@ EXPORT void simple_message (const char * title, const char * text, QMessageBox::
 /* translate GTK+ accelerators and also handle dgettext() */
 EXPORT QString translate_str (const char * str, const char * domain)
 {
+    /* handle null and empty strings */
+    if (! str || ! str[0])
+        return QString (str);
+
     /* translate the GTK+ accelerator (_) into a Qt accelerator (&) */
     return QString (dgettext (domain, str)).replace ('_', '&');
 }
